#pragma once

/*
MIT License
Copyright (c) 2024 - 2025 René Amthor (tobid7)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
 */

// This file is generated by lazyvec
#include <pd/core/common.hpp>

namespace PD {
template <typename T>
class vec3 {
 public:
  T x;
  T y;
  T z;

  vec3() : x(0), y(0), z(0) {}
  template <typename T1>
  explicit vec3(T1 v) {
    x = (T)v;
    y = (T)v;
    z = (T)v;
  }

  template <typename T1>
  explicit vec3(vec3<T1> v) {
    x = (T)v.x;
    y = (T)v.y;
    z = (T)v.z;
  }

  vec3(T x, T y, T z) : x(x), y(y), z(z) {}

  template <typename T1>
  vec3<T>& operator+=(T1 v) {
    x += (T)v;
    y += (T)v;
    z += (T)v;
    return *this;
  }

  template <typename T1>
  vec3<T>& operator+=(const vec3<T1>& v) {
    x += (T)v.x;
    y += (T)v.y;
    z += (T)v.z;
    return *this;
  }

  template <typename T1>
  vec3<T> operator+(T1 v) const {
    return vec3<T>(x + (T)v, y + (T)v, z + (T)v);
  }

  template <typename T1>
  vec3<T> operator+(const vec3<T1>& v) const {
    return vec3<T>(x + (T)v.x, y + (T)v.y, z + (T)v.z);
  }

  template <typename T1>
  vec3<T>& operator-=(T1 v) {
    x -= (T)v;
    y -= (T)v;
    z -= (T)v;
    return *this;
  }

  template <typename T1>
  vec3<T>& operator-=(const vec3<T1>& v) {
    x -= (T)v.x;
    y -= (T)v.y;
    z -= (T)v.z;
    return *this;
  }

  template <typename T1>
  vec3<T> operator-(T1 v) const {
    return vec3<T>(x - (T)v, y - (T)v, z - (T)v);
  }

  template <typename T1>
  vec3<T> operator-(const vec3<T1>& v) const {
    return vec3<T>(x - (T)v.x, y - (T)v.y, z - (T)v.z);
  }

  template <typename T1>
  vec3<T>& operator*=(T1 v) {
    x *= (T)v;
    y *= (T)v;
    z *= (T)v;
    return *this;
  }

  template <typename T1>
  vec3<T>& operator*=(const vec3<T1>& v) {
    x *= (T)v.x;
    y *= (T)v.y;
    z *= (T)v.z;
    return *this;
  }

  template <typename T1>
  vec3<T> operator*(T1 v) const {
    return vec3<T>(x * (T)v, y * (T)v, z * (T)v);
  }

  template <typename T1>
  vec3<T> operator*(const vec3<T1>& v) const {
    return vec3<T>(x * (T)v.x, y * (T)v.y, z * (T)v.z);
  }

  template <typename T1>
  vec3<T>& operator/=(T1 v) {
    x /= (T)v;
    y /= (T)v;
    z /= (T)v;
    return *this;
  }

  template <typename T1>
  vec3<T>& operator/=(const vec3<T1>& v) {
    x /= (T)v.x;
    y /= (T)v.y;
    z /= (T)v.z;
    return *this;
  }

  template <typename T1>
  vec3<T> operator/(T1 v) const {
    return vec3<T>(x / (T)v, y / (T)v, z / (T)v);
  }

  template <typename T1>
  vec3<T> operator/(const vec3<T1>& v) const {
    return vec3<T>(x / (T)v.x, y / (T)v.y, z / (T)v.z);
  }

  vec3 operator-() const { return vec3(-x, -y, -z); }

  bool operator==(const vec3& v) const {
    return x == v.x && y == v.y && z == v.z;
  }
  bool operator!=(const vec3& v) const { return !(*this == v); }

  double Len() const { return std::sqrt(SqLen()); }
  double SqLen() const { return x * x + y * y + z * z; }

  void SwapXY() {
    T t = x;
    x = y;
    y = t;
  }
  void SwapXZ() {
    T t = x;
    x = z;
    z = t;
  }
  void SwapYZ() {
    T t = y;
    y = z;
    z = t;
  }
};
using fvec3 = vec3<float>;
using dvec3 = vec3<double>;
using ivec3 = vec3<int>;
}  // namespace PD
